from __future__ import annotations

import os
import sys
from dataclasses import dataclass
from textwrap import dedent
from dotenv import load_dotenv

import httpx

from pydantic import BaseModel
from pydantic_ai import Agent
from pydantic_ai.models.gemini import GeminiModel
from pydantic_ai.providers.google_gla import GoogleGLAProvider

load_dotenv()

GEMINI_KEY = os.getenv("GEMINI_API_KEY")
if not GEMINI_KEY:
    print("错误: 未设置 GEMINI_API_KEY 环境变量。请在 .env 文件中添加该变量。")
    print("Example: GEMINI_API_KEY=your_api_key_here")
    sys.exit(1)

PROXY_URL = os.getenv("PROXY_URL")
if not PROXY_URL:
    print("错误: 未设置 PROXY_URL 环境变量。请在 .env 文件中添加该变量。")
    print("Example: PROXY_URL=http://your_proxy_url_here")
    sys.exit(1)

proxies = {
    "http://": PROXY_URL,
    "https://": PROXY_URL,
}

custom_httpx_client = httpx.AsyncClient(
    mounts={
        "http://": httpx.AsyncHTTPTransport(proxy=PROXY_URL),
        "https://": httpx.AsyncHTTPTransport(proxy=PROXY_URL),
    },
    timeout=60.0,
)

SYSTEM_PROMPT = dedent(
    """\
    You are a professional developer specializing in shell commands.
    Your task is to generate the correct shell commands based on the 
    user's request.
    IMPORTANT: ALWAYS USE THE SAME LANGUAGE AS THE USER PROMPT IN 
    YOUR RESPONSE.
     Process:
    1. Think Aloud: Use the `think` function to explain your reasoning. 
    Justify why you chose a particular command, considering efficiency,
    safety, and best practices.
    2. Provide the Final Command: Use the `answer` function to present
    the final shell command concisely.
"""
)


@dataclass
class Answer:
    success: bool
    cmd: str | None
    failure: str | None


provider = GoogleGLAProvider(api_key=GEMINI_KEY, http_client=custom_httpx_client)
model = GeminiModel("gemini-2.0-flash", provider=provider)
agent = Agent(
    model=model,
    system_prompt=SYSTEM_PROMPT,
    output_type=Answer,
)


@agent.tool_plain
def think(s: str) -> None:
    """Communicate your thought process to the user.
    Args:
        s (str): A description of your reasoning or decision-making process.
    """
    print(f"(AI Thinking): {s}\n")


@agent.tool_plain
def answer(success: bool, cmd: str | None, failure: str | None) -> Answer:
    """Provide the final shell command or explain why it couldn't be generated.
    Args:
        success (bool): Indicates whether a shell command was successfully generated.
        cmd (str | None): The generated shell command if `success` is True.
            # 如果成功生成了shell命令，cmd是一个单行的命令。
            It must be a single-line command. If `success` is False, this should be None.
            # 如果未能成功生成命令，cmd应为None。
        failure (str | None): If `success` is False, provide a reason why the command
            could not be generated. If `success` is True, this should be None.
            # 如果未能成功生成命令，提供失败的原因。如果成功生成了命令，则此参数应为None。
    Returns:
        Answer: A structured response that will be processed for the user.
        # 返回一个结构化的响应，供用户处理。
    """
    # 返回Answer对象
    return Answer(success, cmd, failure)


def main() -> None:
    user_prompt = "".join(sys.argv[1:])
    if len(user_prompt) == 0:
        print("No prompts")
        sys.exit(1)

    # print(f"HTTP_PROXY env var: {os.environ.get('HTTP_PROXY')}")
    # print(f"HTTPS_PROXY env var: {os.environ.get('HTTPS_PROXY')}")

    resp = agent.run_sync(user_prompt)
    answer = resp.output
    if answer.success and answer.cmd is not None:
        print(f"(AI Answer): {answer.cmd}")
        y_or_n = input("Execute? Y/N: ")
        if y_or_n in ["y", "Y"]:
            os.system(answer.cmd)
    else:
        print(f"(AI Answer): {answer.failure}")
        print("Generate failed")


if __name__ == "__main__":
    main()
